from PyQt5.QtWebEngineWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *

from functools import partial
import collections
import requests
import hashlib
import base64
import json
import pyscp
import sys
import os

from skippy.utils.language import Translator
from skippy.utils.preview import Preview
from skippy.utils.logger import log
import skippy.utils.critical
import skippy.utils.elements
import skippy.utils.profile
import skippy.config


class MenusWidget(QWidget):
    def __init__(self, parent=None):
        super(MenusWidget, self).__init__(parent)
        self.parent = parent
        menuBar = parent.menuBar()

        self.action_list = {}

        self.contextMenu = self.addMenu()

        file_menu = self.addMenu(Translator.translate("MENU_BAR_FILE_MENU"), menuBar)
        file_toolbar = self.addToolbar(Translator.translate("MENU_BAR_FILE_TOOLBAR"))

        self.addAction(
            Translator.translate("MENU_BAR_NEW_ACTION_NAME"),
            Translator.translate("MENU_BAR_NEW_ACTION_STATUS_TIP"),
            lambda: parent.tab.new_tab(),
            "new",
            file_menu,
            file_toolbar,
            self.contextMenu,
        )
        self.addAction(
            Translator.translate("MENU_BAR_OPEN_ACTION_NAME"),
            Translator.translate("MENU_BAR_OPEN_ACTION_STATUS_TIP"),
            parent.download,
            "open",
            file_menu,
            file_toolbar,
        )
        self.addAction(
            Translator.translate("MENU_BAR_UPLOAD_ACTION_NAME"),
            Translator.translate("MENU_BAR_UPLOAD_ACTION_STATUS_TIP"),
            lambda: parent.upload(parent.tab.tabs.currentWidget()),
            "upload",
            file_menu,
            file_toolbar,
        )
        self.addAction(
            Translator.translate("MENU_BAR_UPLOAD_AS_ACTION_NAME"),
            Translator.translate("MENU_BAR_UPLOAD_AS_ACTION_STATUS_TIP"),
            lambda: parent.upload_as(parent.tab.tabs.currentWidget()),
            "upload_as",
            file_menu,
            file_toolbar,
        )
        self.addAction(
            Translator.translate("MENU_BAR_CLOSE_ACTION_NAME"),
            Translator.translate("MENU_BAR_CLOSE_ACTION_STATUS_TIP"),
            lambda: parent.tab.remove_tab(parent.tab.tabs.currentIndex),
            "close",
            file_menu,
            file_toolbar,
        )

        self.addSeparator(file_menu, self.contextMenu)

        self.addAction(
            Translator.translate("MENU_BAR_LOAD_SESSION_ACTION_NAME"),
            Translator.translate("MENU_BAR_LOAD_SESSION_ACTION_STATUS_TIP"),
            parent.load_session,
            "load_session",
            file_menu,
        )
        self.addAction(
            Translator.translate("MENU_BAR_SAVE_SESSION_ACTION_NAME"),
            Translator.translate("MENU_BAR_SAVE_SESSION_ACTION_STATUS_TIP"),
            parent.save_session,
            "save_session",
            file_menu,
        )

        self.addSeparator(file_menu)

        self.addAction(
            Translator.translate("MENU_BAR_EXIT_ACTION_NAME"),
            Translator.translate("MENU_BAR_EXIT_ACTION_STATUS_TIP"),
            parent.close,
            "exit",
            file_menu,
        )

        edit_menu = self.addMenu(Translator.translate("MENU_BAR_EDIT_MENU"), menuBar)
        edit_toolbar = self.addToolbar(Translator.translate("MENU_BAR_EDIT_TOOLBAR"))

        self.addAction(
            Translator.translate("MENU_BAR_UNDO_ACTION_NAME"),
            Translator.translate("MENU_BAR_UNDO_ACTION_STATUS_TIP"),
            lambda: parent.tab.tabs.currentWidget().findChild(QPlainTextEdit).undo(),
            "undo",
            edit_menu,
            edit_toolbar,
            self.contextMenu,
        )
        self.addAction(
            Translator.translate("MENU_BAR_REDO_ACTION_NAME"),
            Translator.translate("MENU_BAR_REDO_ACTION_STATUS_TIP"),
            lambda: parent.tab.tabs.currentWidget().findChild(QPlainTextEdit).redo(),
            "redo",
            edit_menu,
            edit_toolbar,
            self.contextMenu,
        )

        self.addSeparator(edit_menu, self.contextMenu)

        self.addAction(
            Translator.translate("MENU_BAR_CUT_ACTION_NAME"),
            Translator.translate("MENU_BAR_CUT_ACTION_STATUS_TIP"),
            lambda: parent.tab.tabs.currentWidget().findChild(QPlainTextEdit).cut(),
            "cut",
            edit_menu,
            edit_toolbar,
            self.contextMenu,
        )
        self.addAction(
            Translator.translate("MENU_BAR_COPY_ACTION_NAME"),
            Translator.translate("MENU_BAR_COPY_ACTION_STATUS_TIP"),
            lambda: parent.tab.tabs.currentWidget().findChild(QPlainTextEdit).copy(),
            "copy",
            edit_menu,
            edit_toolbar,
            self.contextMenu,
        )
        self.addAction(
            Translator.translate("MENU_BAR_PASTE_ACTION_NAME"),
            Translator.translate("MENU_BAR_PASTE_ACTION_STATUS_TIP"),
            lambda: parent.tab.tabs.currentWidget().findChild(QPlainTextEdit).paste(),
            "paste",
            edit_menu,
            edit_toolbar,
            self.contextMenu,
        )
        self.addAction(
            Translator.translate("MENU_BAR_SELECT_ALL_ACTION_NAME"),
            Translator.translate("MENU_BAR_SELECT_ALL_ACTION_STATUS_TIP"),
            lambda: parent.tab.tabs.currentWidget()
            .findChild(QPlainTextEdit)
            .selectAll(),
            "redo",
            edit_menu,
            self.contextMenu,
        )

        self.addSeparator(edit_menu, self.contextMenu)

        self.addAction(
            Translator.translate("MENU_BAR_PREVIEW_ACTION_NAME"),
            Translator.translate("MENU_BAR_PREVIEW_ACTION_STATUS_TIP"),
            lambda: Previewer(parent.tab.tabs.currentWidget().data, parent),
            "preview",
            edit_menu,
            edit_toolbar,
            self.contextMenu,
        )

        element_menu = self.addMenu(
            Translator.translate("MENU_BAR_INSERT_MENU"), edit_menu, self.contextMenu
        )

        for element in skippy.utils.elements.Elements.elements:
            name = element.__alias__ if element.__alias__ else element.__name__
            self.addAction(
                name,
                name,
                partial(ElementGenerator, element, parent),
                None,
                element_menu,
            )

        settings_menu = self.addMenu(
            Translator.translate("MENU_BAR_SETTINGS_MENU"), menuBar
        )

        self.addAction(
            Translator.translate("MENU_BAR_TOGGLE_THEME_ACTION_NAME"),
            Translator.translate("MENU_BAR_TOGGLE_THEME_ACTION_STATUS_TIP"),
            parent.toggle_theme,
            "toggle",
            settings_menu,
        )

        language_menu = self.addMenu(
            Translator.translate("MENU_BAR_LANGUAGES_MENU"), settings_menu
        )

        for lang in Translator.languages():
            name = Translator.getLangName(lang)
            self.addAction(
                name, name, partial(parent.updateTranslate, lang), None, language_menu
            )

        self.addSeparator(settings_menu)

        self.addAction(
            Translator.translate("MENU_BAR_LOGIN_ACTION_NAME"),
            Translator.translate("MENU_BAR_LOGIN_ACTION_STATUS_TIP"),
            parent.login,
            "login",
            settings_menu,
        )
        self.addAction(
            Translator.translate("MENU_BAR_LOGOUT_ACTION_NAME"),
            Translator.translate("MENU_BAR_LOGOUT_ACTION_STATUS_TIP"),
            parent.logout,
            "logout",
            settings_menu,
        )

    def addMenu(self, label="", *widgets):
        menu = QMenu(label, self.parent)
        for widget in widgets:
            widget.addMenu(menu)

        return menu

    def addToolbar(self, label):
        toolbar = QToolBar(label, self.parent)
        toolbar.setIconSize(QSize(20, 20))
        self.parent.addToolBar(toolbar)

        return toolbar

    def addSeparator(self, *menus):
        for menu in menus:
            menu.addSeparator()

    def addAction(self, label, statusTip, func, img, *menus):
        action = QAction(
            label,
            self,
        )

        if img:
            action.setIcon(
                QIcon(
                    os.path.join(
                        skippy.config.ASSETS_FOLDER,
                        self.parent.settings.value("mode", "light"),
                        f"{img}.png",
                    )
                )
            )
            self.action_list[action] = img
        action.setStatusTip(statusTip)
        action.triggered.connect(func)
        for menu in menus:
            menu.addAction(action)

        return action


class LoginStatusWidget(QWidget):
    def __init__(self, parent=None):
        super(LoginStatusWidget, self).__init__(parent)
        self.parent = parent
        self.setGeometry(300, 300, 300, 50)

        self.layout = QVBoxLayout(self)

        self.label = QLabel("", self)
        self.label.setFont(QFont("Arial", 10))

        self.layout.addWidget(self.label)
        self.setLayout(self.layout)

        username = skippy.utils.profile.Profile.load()[0]
        if username != "" and username != None:
            self.label.setText(
                Translator.translate("SIGNED_IN_AS_LABEL").format(username)
            )
        else:
            self.label.setText("")

    def set_text(self, text):
        self.label.setText(Translator.translate("SIGNED_IN_AS_LABEL").format(username))


class ProjectList(QWidget):
    def __init__(self, parent=None):
        super(ProjectList, self).__init__(parent)
        self.parent = parent

        self.layout = QVBoxLayout(self)

        self.tabs = QTabWidget()
        self.tabs.setStyleSheet("QTabBar{font-family: Arial; font-size:10pt;}")
        self.tabs.setTabsClosable(True)
        self.tabs.setMovable(True)
        self.tabs.resize(300, 200)
        self.setAcceptDrops(True)

        self.tabs.tabCloseRequested.connect(self.remove_tab)

        self.layout.addWidget(self.tabs)
        self.setLayout(self.layout)

        self.new_tab()

    @skippy.utils.critical.critical
    def new_tab(self, title="New Tab", source="", tags=[], files={}, parent=None):
        tab = QWidget()
        self.tabs.addTab(tab, title)
        tab.layout = QVBoxLayout(tab)

        if type(files) != dict:
            files = {}

        tab.data = {
            "title": title,
            "source": source,
            "tags": tags,
            "files": files,
            "parent": parent,
        }

        def setData(param, data):
            tab.data[param] = data
            self.parent.session.save()

        title_box = QLineEdit()
        title_box.setText(title)
        title_box.textEdited.connect(
            lambda: self.tabs.setTabText(self.tabs.currentIndex(), title_box.text())
        )
        title_box.textEdited.connect(lambda text: setData("title", text))
        title_box.textEdited.connect(self.parent.update_title)
        title_box.setStyleSheet("QLineEdit{font-family: Arial; font-size:11pt;}")

        editor = CustomPlainTextEdit(self.parent, self)
        editor.setPlainText(source)
        editor.textChanged.connect(lambda: setData("source", editor.toPlainText()))
        editor.setStyleSheet("QPlainTextEdit{font-family: Arial; font-size:11pt;}")

        tags_box = QLineEdit()
        if isinstance(tags, collections.Iterable):
            tags_box.setText(" ".join(tags))
        tags_box.textChanged.connect(lambda text: setData("tags", text.split(" ")))
        tags_box.setStyleSheet("QLineEdit{font-family: Arial; font-size:11pt;}")

        files_button = QPushButton(Translator.translate("FILES_BUTTON"), self)
        files_button.clicked.connect(self.files_dialog)

        tab.layout.addWidget(title_box)
        tab.layout.addWidget(editor)
        tab.layout.addWidget(tags_box)
        tab.layout.addWidget(files_button)
        tab.setLayout(tab.layout)

        log.debug(f"New tab is created with title: {title}")

        self.tabs.setCurrentIndex(self.tabs.count() - 1)

    @skippy.utils.critical.critical
    def remove_tab(self, index):
        self.tabs.removeTab(index)
        if self.tabs.count() == 0:
            self.new_tab()

    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
            self.dadDialog = DragAndDropWidget(self)
        else:
            event.ignore()

    def dragLeaveEvent(self, event):
        event.accept()
        self.dadDialog.close()

    def dropEvent(self, event):
        self.dadDialog.close()
        widget = self.tabs.currentWidget()
        files = [u.toLocalFile() for u in event.mimeData().urls()]
        self.thread = QThread()
        self.worker = UploadFiles(files)
        self.worker.moveToThread(self.thread)
        self.thread.started.connect(self.worker.run)
        self.worker.finished.connect(self.thread.quit)
        self.worker.finished.connect(self.parent.session.save)
        self.worker.progress.connect(self.upload_file)
        self.thread.start()

    def upload_file(self, file):
        filename = file[0]
        source = file[1]
        widget = self.tabs.currentWidget()
        widget.data["files"][filename] = (
            str(base64.b64encode(source)).replace("b'", "").replace("'", "")
        )

    def files_dialog(self):
        widget = self.tabs.currentWidget()
        fDialog = FilesDialog(widget, self)


class CustomPlainTextEdit(QPlainTextEdit):
    def __init__(self, mainWindow, parent=None):
        super(CustomPlainTextEdit, self).__init__(parent)
        self.mainWindow = mainWindow

    def contextMenuEvent(self, event):
        self.mainWindow.menus.contextMenu.exec_(self.mapToGlobal(event.pos()))


class DragAndDropWidget(QWidget):
    CLOSE = pyqtSignal()

    def __init__(self, parent=None):
        super(DragAndDropWidget, self).__init__(parent)
        self.width, self.height = (
            parent.frameGeometry().width(),
            parent.frameGeometry().height(),
        )

        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

        self.fillColor = QColor(30, 30, 30, 120)
        self.penColor = QColor("#333333")

        self.popup_fillColor = QColor(240, 240, 240, 255)
        self.popup_penColor = QColor(200, 200, 200, 255)

        self.CLOSE.connect(self.close)

        self.move(0, 0)
        self.resize(self.width, self.height)
        self.show()

    def resizeEvent(self, event):
        popup_width = 300
        popup_height = 120
        ow = int(self.width / 2 - popup_width / 2)
        oh = int(self.height / 2 - popup_height / 2)

    def paintEvent(self, event):
        s = self.size()
        qp = QPainter()
        qp.begin(self)
        qp.setRenderHint(QPainter.Antialiasing, True)
        qp.setPen(self.penColor)
        qp.setBrush(self.fillColor)
        qp.drawRect(0, 0, self.width, self.height)

        qp.setPen(self.popup_penColor)
        qp.setBrush(self.popup_fillColor)
        popup_width = 300
        popup_height = 120
        ow = int(self.width / 2 - popup_width / 2)
        oh = int(self.height / 2 - popup_height / 2)
        qp.drawRoundedRect(ow, oh, popup_width, popup_height, 5, 5)

        font = QFont()
        font.setPixelSize(18)
        font.setBold(True)
        qp.setFont(font)
        qp.setPen(QColor(70, 70, 70))
        tolw, tolh = 80, -5
        qp.drawText(
            ow + int(popup_width / 2) - tolw,
            oh + int(popup_height / 2) - tolh,
            "Drop File.",
        )

        qp.end()

    def _close(self):
        self.CLOSE.emit()


class FilesDialog(QDialog):
    def __init__(self, widget, parent=None):
        super(FilesDialog, self).__init__(parent)
        self.parent = parent
        self.layout = QVBoxLayout(self)

        self.label = QLabel(Translator.translate("FILES_DIALOG_FILES_LIST_LABEL"), self)
        self.label.setFont(QFont("Arial", 10))

        self.fileWidget = FilesWidget(widget, self)

        self.layout.addWidget(self.label, alignment=Qt.AlignCenter)

        if self.fileWidget.layout.count():
            self.layout.addWidget(self.fileWidget, alignment=Qt.AlignCenter)
        else:
            self.empty_files_label = QLabel(
                Translator.translate("FILES_DIALOG_EMPTY_FILES_LIST_LABEL"), self
            )
            self.layout.addWidget(self.empty_files_label, alignment=Qt.AlignCenter)

        self.setLayout(self.layout)

        self.setWindowTitle(f"skippy - {skippy.config.version}")
        self.move(300, 300)
        self.resize(200, 100)
        self.show()


class FilesWidget(QWidget):
    def __init__(self, widget, parent=None):
        super(FilesWidget, self).__init__(parent)
        self.parent = parent

        self.layout = QGridLayout(self)

        files = widget.data["files"]

        pos = []
        for i in range(int(len(files) / 3)):
            for j in range(3):
                pos.append((i, j))
        if len(files) % 3 != 0:
            for i in range(len(files) % 3):
                pos.append((int(len(files) / 3), i))

        for e, p in zip(list(files.keys()), pos):
            self.layout.addWidget(FileWidget(e, widget, self), p[0], p[1])

        self.setLayout(self.layout)


class FileWidget(QWidget):
    del_widget = pyqtSignal(str)

    def __init__(self, title, widget, parent=None):
        super(FileWidget, self).__init__(parent)
        self.parent = parent
        self.widget = widget
        # self.setStyleSheet('background-color: rgb(225,225,225); margin:5px; border:1px solid rgb(0, 0, 0);')

        self.title = title

        self.layout = QGridLayout(self)

        self.lineEdit = QLineEdit(self)
        self.lineEdit.setText(title)
        self.lineEdit.setFixedSize(200, 40)
        self.lineEdit.textEdited.connect(self.set_title)

        self.remove_button = QPushButton("X", self)
        self.remove_button.clicked.connect(self.remove_file)

        self.layout.addWidget(self.lineEdit, 0, 0)
        self.layout.addWidget(self.remove_button, 0, 1)
        self.setLayout(self.layout)

    @skippy.utils.critical.critical
    def set_title(self, text):
        self.widget.data["files"][text] = self.widget.data["files"][self.title]
        log.debug(f"file ")
        del self.widget.data["files"][self.title]
        self.title = text

    @skippy.utils.critical.critical
    def remove_file(self, *args):
        self.widget.data["files"].pop(self.title)


class LoginDialog(QDialog):
    def __init__(self, parent=None):
        super(LoginDialog, self).__init__(parent)
        self.parent = parent

        self.auth = False

        self.layout = QVBoxLayout(self)

        self.label = QLabel(
            Translator.translate("DIALOG_SIGN_IN_TO_WIKIDOT_LABEL"), self
        )
        self.login_box = QLineEdit()
        self.login_box.setPlaceholderText(
            Translator.translate("DIALOG_LOGIN_PLACEHOLDER")
        )
        self.password_box = QLineEdit()
        self.password_box.setEchoMode(QLineEdit.Password)
        self.password_box.setPlaceholderText(
            Translator.translate("DIALOG_PASSWORD_PLACEHOLDER")
        )
        self.button = QPushButton(Translator.translate("DIALOG_OK_BUTTON"), self)

        self.button.clicked.connect(self.login)

        self.layout.addWidget(self.label, alignment=Qt.AlignCenter)
        self.layout.addWidget(self.login_box)
        self.layout.addWidget(self.password_box)
        self.layout.addWidget(self.button, alignment=Qt.AlignRight)

        self.setLayout(self.layout)

        self.setWindowTitle(f"skippy - {skippy.config.version}")
        self.move(300, 300)
        self.resize(200, 100)
        self.show()

    def login(self):
        login = self.login_box.text()
        password = self.password_box.text()
        skippy.utils.profile.Profile.save(login, password)
        self.parent.show()
        self.auth = True
        log.debug(f"Login as {login}")
        self.parent.login_status.set_text(login)
        self.close()

    def closeEvent(self, event):
        if not self.auth:
            sys.exit()


class UploadDialog(QDialog):
    def __init__(self, widget, parent=None):
        super(UploadDialog, self).__init__(parent)
        self.widget = widget
        self.layout = QVBoxLayout(self)

        self.sites = {}

        def addSites(sites):
            if sites[0] in self.sites:
                addSites((sites[0] + "(1)", sites[1]))
            else:
                self.sites[sites[0]] = sites[1]
                self.site_box.addItem(sites[0])

        self.label = QLabel(Translator.translate("DIALOG_ENTER_PAGE_LABEL"), self)
        self.site_box = QComboBox(self)
        self.thread = QThread()
        self.worker = GetSites()
        self.worker.moveToThread(self.thread)
        self.thread.started.connect(self.worker.run)
        self.worker.finished.connect(self.thread.quit)
        self.worker.progress.connect(addSites)
        self.page_box = QLineEdit()
        self.page_box.setPlaceholderText(
            Translator.translate("DIALOG_PAGE_BOX_PLACEHOLDER")
        )
        self.comment_box = QLineEdit()
        self.comment_box.setText(Translator.translate("DIALOG_COMMENT_BOX_TEXT"))
        self.comment_box.setPlaceholderText(
            Translator.translate("DIALOG_COMMENT_BOX_PLACEHOLDER")
        )
        self.button = QPushButton(Translator.translate("DIALOG_OK_BUTTON"), self)

        self.button.clicked.connect(self.upload)

        self.layout.addWidget(self.label, alignment=Qt.AlignCenter)
        self.layout.addWidget(self.site_box)
        self.layout.addWidget(self.page_box)
        self.layout.addWidget(self.comment_box)
        self.layout.addWidget(self.button, alignment=Qt.AlignRight)

        self.setLayout(self.layout)

        self.setWindowTitle(f"skippy - {skippy.config.version}")
        self.move(300, 300)
        self.resize(200, 100)

        self.thread.start()
        self.show()

    @skippy.utils.critical.critical
    def upload(self, *args):
        site = str(self.site_box.currentText())
        page = self.page_box.text()
        comment = self.comment_box.text()
        wiki = self.sites[site]
        profile = skippy.utils.profile.Profile.load()
        wiki.auth(profile[0], profile[1])
        p = wiki(page)
        p.edit(
            source=self.widget.data["source"],
            title=self.widget.data["title"],
            comment=comment,
        )
        self.widget.data["parent"] = [
            self.sites[site].site.replace("http://", ""),
            page,
        ]
        p.set_tags(self.widget.data["tags"])

        for file in self.widget.data["files"]:
            try:
                p.upload(file, base64.b64decode(self.widget.data["files"][file]))
                log.debug(f"File {file} is uploaded")
            except RuntimeError as e:
                log.debug(f"RuntimeError({str(e)}): File {file} isn't uploaded")
        for file in p.files:
            if file.name not in self.widget.data["files"]:
                p.remove_file(file.name)
                log.debug(f"File {file.name} is deleted")
        log.debug(
            f"""Upload "{self.widget.data["title"]}" to "{"/".join(self.widget.data["parent"])}" """
        )
        self.close()

    def closeEvent(self, e):
        self.worker.stop()
        e.accept()


class DownloadDialog(QDialog):
    def __init__(self, parent=None):
        super(DownloadDialog, self).__init__(parent)
        self.parent = parent
        self.layout = QVBoxLayout(self)

        self.sites = {}

        def addSites(sites):
            if sites[0] in self.sites:
                addSites((sites[0] + "(1)", sites[1]))
            else:
                self.sites[sites[0]] = sites[1]
                self.site_box.addItem(sites[0])

        self.label = QLabel(Translator.translate("DIALOG_ENTER_PAGE_LABEL"), self)
        self.site_box = QComboBox(self)
        self.thread = QThread()
        self.worker = GetSites()
        self.worker.moveToThread(self.thread)
        self.thread.started.connect(self.worker.run)
        self.worker.finished.connect(self.thread.quit)
        self.worker.progress.connect(addSites)
        self.page_box = QLineEdit()
        self.page_box.setPlaceholderText(
            Translator.translate("DIALOG_PAGE_BOX_PLACEHOLDER")
        )
        self.button = QPushButton(Translator.translate("DIALOG_OK_BUTTON"), self)

        self.button.clicked.connect(self.download)

        self.layout.addWidget(self.label, alignment=Qt.AlignCenter)
        self.layout.addWidget(self.site_box)
        self.layout.addWidget(self.page_box)
        self.layout.addWidget(self.button, alignment=Qt.AlignRight)

        self.setLayout(self.layout)

        self.setWindowTitle(f"skippy - {skippy.config.version}")
        self.move(300, 300)
        self.resize(200, 100)

        self.thread.start()
        self.show()

    @skippy.utils.critical.critical
    def download(self, *args):
        site = str(self.site_box.currentText())
        page = self.page_box.text()

        wiki = self.sites[site]
        p = wiki(page)

        url = [self.sites[site].site.replace("http://", ""), page]

        files = {}
        for i in p.files:
            with requests.get(i.url) as res:
                files[i.name] = (
                    str(base64.b64encode(res.content))
                    .replace("b'", "")
                    .replace("'", "")
                )

        self.parent.tab.new_tab(p.title, p.source, list(p.tags), files, url)
        self.parent.session.save()
        log.debug(f"""Download page from "{self.sites[site].site}" """)
        self.close()

    def closeEvent(self, e):
        self.worker.stop()
        e.accept()


class Previewer(QDialog):
    def __init__(self, data, parent=None):
        super(Previewer, self).__init__(parent)
        self.html = Preview(data).process()

        self.layout = QVBoxLayout(self)

        self.webEngineView = QWebEngineView()
        self.webEngineView.load(QUrl.fromLocalFile(self.html))

        self.layout.addWidget(self.webEngineView)

        self.setLayout(self.layout)

        self.setWindowTitle(f"skippy - {skippy.config.version}")
        self.move(parent.x(), parent.y())
        self.resize(parent.width(), parent.height())
        self.setWindowState(parent.windowState())

        self.show()


class ElementGenerator(QDialog):
    def __init__(self, element, parent=None):
        super(ElementGenerator, self).__init__(parent)
        self.element = element()
        self.parent = parent

        self.layout = QVBoxLayout(self)

        self.element_data = QLabel(
            f"{self.element.__alias__} - {self.element.__description__}", self
        )
        self.element_data.setWordWrap(True)

        self.layout.addWidget(self.element_data)
        self.layout.addWidget(QHLine())

        self.scroll = QScrollArea()
        self.widget = QWidget()
        self.vbox = QVBoxLayout()
        self.fields = {}
        for field in self.element.requiredFieldList:
            label = QLabel(field.name if field.name else field.tag, self)

            line = QPlainTextEdit(self)
            line.setFixedHeight(50)
            line.setPlaceholderText(field.description)
            line.textChanged.connect(self.updatePreview)
            self.fields[field.tag] = line

            self.vbox.addWidget(label)
            self.vbox.addWidget(line)

        self.widget.setLayout(self.vbox)

        self.scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll.setWidgetResizable(True)
        self.scroll.setWidget(self.widget)

        self.layout.addWidget(self.scroll)

        self.preview = QPlainTextEdit(self)
        self.preview.setReadOnly(True)

        self.layout.addWidget(self.preview)

        self.button = QPushButton(Translator.translate("DIALOG_GENERATE_BUTTON"), self)
        self.button.clicked.connect(self.process)
        self.layout.addWidget(self.button, alignment=Qt.AlignRight)

        self.setLayout(self.layout)

        self.updatePreview()

        self.setWindowTitle(
            f"{self.element.__alias__} | skippy - {skippy.config.version}"
        )
        self.move(300, 300)
        self.resize(500, 400)

        self.show()

    @skippy.utils.critical.critical
    def updatePreview(self, *args):
        self.preview.setPlainText(self.generate)

    @skippy.utils.critical.critical
    def process(self, *args):
        self.parent.tab.tabs.currentWidget().findChild(QPlainTextEdit).insertPlainText(
            self.generate
        )
        self.close()

    @property
    def generate(self):
        data = {}
        for field in self.fields:
            data[field] = self.fields[field].toPlainText()
        return self.element.generate(**data)


class QHLine(QFrame):
    def __init__(self):
        super(QHLine, self).__init__()
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)


class GetSites(QObject):
    progress = pyqtSignal(tuple)
    finished = pyqtSignal()
    cache = []
    run = True

    def run(self):
        sites = list(pyscp.wikidot.User(skippy.utils.profile.Profile.load()[0]).member)
        for data in self.cache:
            self.progress.emit((data[0], data[1]))
        if len(self.cache) < len(sites):
            for site in sites[len(self.cache) :]:
                if self.run:
                    self.progress.emit((site.title, site))
                    self.cache.append((site.title, site))
                    log.debug(f"Get sites: {site.title}")
        self.finished.emit()

    def stop(self):
        self.run = False


class UploadFiles(QObject):
    progress = pyqtSignal(tuple)
    finished = pyqtSignal()

    def __init__(self, files):
        super(UploadFiles, self).__init__()
        self.files = files

    def run(self):
        log.debug("File loaded started...")
        for file in self.files:
            if os.path.isfile(file):
                filename = os.path.basename(file)
                with open(file, "rb") as f:
                    source = f.read()
                log.debug(f"Load {file}")
                self.progress.emit((filename, source))
        self.finished.emit()



from skippy.utils.logger import log

from abc import ABCMeta, abstractmethod
import unicodedata
import tempfile
import pyscp
import html
import re


class Preview:
    def __init__(self, data):
        self.data = data

    def process(self):
        preprocess = PreProcessorsHandler(self.data).process()
        htmlprocess = HTMLProcessorsHandler(preprocess, self.data).process()
        postprocess = PostProcessorsHandler(htmlprocess, self.data).process()

        with tempfile.NamedTemporaryFile(
            delete=False, suffix=".html", mode="w", encoding="utf-8"
        ) as tmp:
            tmp.write(postprocess)
            return tmp.name


#################################################
# Base Classes
#################################################
class ProcessorBase(metaclass=ABCMeta):
    pattern = r""

    def __init__(self, source, data):
        self.source = source
        self.data = data

    @property
    def matches(self):
        return re.findall(
            self.pattern,
            self.source,
        )

    @abstractmethod
    def process(self):
        pass


class ProcessorsHandler(metaclass=ABCMeta):
    def __init__(self, source, data):
        self.source = source
        self.data = data
        self.processors = []

    def register(self, processor):
        self.processors.append(processor)

    def process(self):
        for processor in self.processors:
            try:
                self.source = processor(self.source, self.data).process()
            except Exception as e:
                log.error(e, exc_info=True)
        return self.source


#################################################
# ProcessorsHandlers
#################################################
class PreProcessorsHandler(ProcessorsHandler):
    def __init__(self, data):
        super(PreProcessorsHandler, self).__init__(data["source"], data)
        self.register(IncludesProcessor)
        self.register(IftagsProcessor)


class HTMLProcessorsHandler(ProcessorsHandler):
    def __init__(self, source, data):
        super(HTMLProcessorsHandler, self).__init__(source, data)
        self.register(MarkdownProcessor)
        self.register(InsertDataProcessor)
        self.register(ModuleCSSProcessor)

    def process(self):
        self.html = self.processors[0](self.source, self.data).process()
        for processor in self.processors[1:]:
            try:
                self.html = processor(self.source, self.data, self.html).process()
            except Exception as e:
                log.error(e, exc_info=True)
        return self.html


class PostProcessorsHandler(ProcessorsHandler):
    def __init__(self, source, data):
        super(PostProcessorsHandler, self).__init__(source, data)
        self.register(LocalImagesProcessor)
        self.register(HTMLTagsProcessor)


#################################################
# PreProcessors
#################################################
class IncludesProcessor(ProcessorBase):
    pattern = r"(\[\[include\s(?::.+?:|)(?:.+?:|)(?:.+)(?:\s((?:.|\n)+?)|)]])"

    def process(self, iteration=0):
        for include in self.matches:
            path = re.findall(
                r"(:.+?:|)(.+?:|)(.+)",
                include[0].split()[1].replace("]]", ""),
            )[0]
            site = path[0].replace(":", "")
            page = path[1] + path[2]
            try:
                wiki = pyscp.wikidot.Wiki(site)
                p = wiki(page).source
                args = [
                    i[1:] if i.startswith("\n") else i for i in include[1].split("|")
                ]
                if args[0] == "":
                    args = []
                for arg in args:
                    arg = re.findall(
                        r"(?:([\w-]+)(?:(?:\s|)=(?:\s|))((?:.|\n+?)+))",
                        arg,
                    )[0]
                    p = p.replace("{$" + arg[0] + "}", arg[1])
                self.source = self.source.replace(include[0], p)
            except Exception as e:
                log.error(e)
        if (
            re.findall(
                r"(\[\[include(?:(?:.|\n)+?)]])",
                self.source,
            )
            and iteration != 5
        ):
            self.process(iteration + 1)
        return self.source


class IftagsProcessor(ProcessorBase):
    pattern = r"(\[\[iftags (.+?)]]((?:(?:.|\n|\s)+?))\[\[\/iftags]])"

    def process(self):
        for iftag in self.matches:
            match = True
            elem = re.findall(r"(\+.+?(?=\s|$)|-.+?(?=\s|$))", iftag[1])
            for e in elem:
                if e.startswith("+"):
                    if e[1:] not in self.data["tags"]:
                        match = False
                else:
                    if e[1:] in self.data["tags"]:
                        match = False
            if match:
                self.source = self.source.replace(iftag[0], iftag[2])
        return self.source


#################################################
# HTMLProcessors
#################################################
class MarkdownProcessor(ProcessorBase):
    wiki = pyscp.wikidot.Wiki("www.wikidot.com")

    def process(self):
        return self.wiki._module("edit/PagePreviewModule", source=self.source)["body"]


class InsertDataProcessor(ProcessorBase):
    html_base = """
		<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
		<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ru" lang="ru" data-lt-installed="true">
		<head>
		<style type="text/css">
			@import url(http://d3g0gp89917ko0.cloudfront.net/v--3e3a6f7dbcc9/common--theme/base/css/style.css);
			@import url(http://scp-ru.wdfiles.com/local--code/component:theme2/1);
		</style>
		<script type="text/javascript" src="http://d3g0gp89917ko0.cloudfront.net/v--3e3a6f7dbcc9/common--javascript/init.combined.js"></script>
		<script type="text/javascript">
		var URL_HOST = 'www.wikidot.com';
		var URL_DOMAIN = 'wikidot.com';
		var USE_SSL =  true ;
		var URL_STATIC = 'http://d3g0gp89917ko0.cloudfront.net/v--3e3a6f7dbcc9';
		// global request information
		
		var WIKIREQUEST = {};
		WIKIREQUEST.info = {};
		
		WIKIREQUEST.info.domain = "www.wikidot.com";
		WIKIREQUEST.info.siteId = 648902;
		WIKIREQUEST.info.siteUnixName = "www";
		WIKIREQUEST.info.categoryId = 4388020;
		WIKIREQUEST.info.themeId = 192064;
		WIKIREQUEST.info.requestPageName = "start:start";
		OZONE.request.timestamp = 1619763822;
		OZONE.request.date = new Date();
		WIKIREQUEST.info.lang = 'en';
				WIKIREQUEST.info.pageUnixName = "start:start";
		WIKIREQUEST.info.pageId = 22129557;
						WIKIREQUEST.info.lang = "en";
		OZONE.lang = "en";
		var isUAMobile = !!/Android|webOS|iPhone|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
		</script>
		<script type="text/javascript">
	
			require.config({
				baseUrl: URL_STATIC + '/common--javascript',
				paths: {
					'jquery.ui': 'jquery-ui.min',
					'jquery.form': 'jquery.form'
				}
			});
	
		</script>
		<script type="text/javascript" src="http://d3g0gp89917ko0.cloudfront.net/v--3e3a6f7dbcc9/common--javascript/WIKIDOT.combined.js"></script>
		<<MODULE-CSS-PREVIEW>>
		<style>
			body {
				padding: 2em;
			}
		</style>
		</head>
		<body id="html-body">
		<div id="dummy-ondomready-block" style="display:none;"></div>
		<div id="page-title">
			<<TITLE>>
		</div>
		<div id="page-content">
			<<CONTENT>>
		</div>
		<div class="page-tags">
		<span>
			<<TAGS>>
		</span>
		</div>
		</body>
		</html>"""

    def __init__(self, source, data, html):
        super(InsertDataProcessor, self).__init__(source, data)
        self.html = html

    def process(self):
        html = self.html_base.replace("<<TITLE>>", self.data["title"])
        html = html.replace("<<CONTENT>>", self.html)
        html = html.replace(
            "<<TAGS>>",
            " ".join([f"<a href='#'>{tag}</a>" for tag in self.data["tags"]]),
        )
        return html


class ModuleCSSProcessor(ProcessorBase):
    pattern = r"(?:(?:\[\[module) (?:CSS|css)(?:(.+?)|)\]\]\n)((.|\n)+?)(?:\n\[\[\/module\]\])"

    def __init__(self, source, data, html):
        super(ModuleCSSProcessor, self).__init__(source, data)
        self.html = html

    def process(self):
        styles = ""
        for style in self.matches:
            style = unicodedata.normalize("NFKD", style[1])
            styles += f"<style>\n{style}\n</style>\n"
        self.html = self.html.replace("<<MODULE-CSS-PREVIEW>>", styles)
        return self.html


#################################################
# PostProcessor
#################################################
class LocalImagesProcessor(ProcessorBase):
    pattern = r"""(?<=(<img src="))(http://www.wdfiles.com/local--files//)(.+?)(?=")"""

    def process(self):
        files = self.data["files"]
        for img in self.matches:
            img = img[1:]
            if img[1] in files:
                self.source = self.source.replace(
                    img[0] + img[1],
                    f"data:{img[1].split('.')[1]}/;base64,{files[img[1]]}",
                )
        return self.source


class HTMLTagsProcessor(ProcessorBase):
    pattern = r"(\[\[html(?:(?:.+?)| |)]]((?:.|\n|\s)+?)\[\[\/html]])"

    def process(self):
        for htmltags in self.matches:
            self.source = self.source.replace(htmltags[0], html.unescape(htmltags[1]))
        return self.source



import skippy.config

import json
import os

try:
    from collections.abc import namedtuple
except ImportError:
    from collections import namedtuple

Language = namedtuple("Language", "code dict")


class Translator:
    language = None

    @staticmethod
    def languages():
        return [
            os.path.splitext(file)[0]
            for file in os.listdir(skippy.config.LANG_FOLDER)
            if os.path.isfile(os.path.join(skippy.config.LANG_FOLDER, file))
            if os.path.splitext(file)[1] == ".json"
        ]

    @staticmethod
    def getLangName(lang):
        path = os.path.join(skippy.config.LANG_FOLDER, f"{lang}.json")
        with open(path, encoding="utf-8") as f:
            return json.loads(f.read())["LANGUAGE"]

    @classmethod
    def load(cls, lang="en"):
        path = os.path.join(skippy.config.LANG_FOLDER, f"{lang}.json")
        with open(path, encoding="utf-8") as f:
            dictionary = json.loads(f.read())
        cls.language = Language(lang, dictionary)

        return cls.language

    @classmethod
    def translate(cls, context):
        try:
            return cls.language.dict[context]
        except KeyError:
            return context



import setuptools
import skippy

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

with open("README.md", encoding="utf8") as f:
    readme = f.read()

setuptools.setup(
    name="skippy-pad",
    version=skippy.__version__,
    description="Simple. Comfortable. Powerful. notepad for www.scpwiki.com.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/skippy-dev/skippy/",
    author="MrNereof",
    author_email="mrnereof@gmail.com",
    python_requires=">=3.4",
    license="GPLv3",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Other Audience",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords=["scp", "wikidot", "pyscp", "skippy"],
    packages=[
        "skippy",
        "skippy.plugins",
        "skippy.utils",
        "skippy.assets",
        "skippy.lang",
    ],
    install_requires=requirements,
    package_dir={"skippy": "skippy"},
    package_data={
        "skippy": ["assets/*.*", "assets/dark/*.*", "assets/light/*.*", "lang/*.json"]
    },
    entry_points={
        "console_scripts": [
            "skippy = skippy.__main__:run",
        ],
    },
)



from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
import qtmodern.styles
import collections

from skippy.utils.language import Translator
from skippy.utils.logger import log
import skippy.utils.critical
import skippy.utils.session
import skippy.utils.profile
import skippy.config
from skippy.widgets import *

import base64
import pyscp
import json
import os
import sys


class App(QMainWindow):
    def __init__(self, app):
        super(App, self).__init__()
        self.app = app

        self.settings = QSettings("skippy", "skippy")

        Translator.load(self.settings.value("lang", "en"))

        self.tab = ProjectList(self)
        self.tab.tabs.currentChanged.connect(self.update_title)

        self.session = skippy.utils.session.Session(self)
        self.session.load()

        self.menus = MenusWidget(self)

        self.login_status = LoginStatusWidget(self)
        self.login_status.move(500, -14)

        self.setCentralWidget(self.tab)

        self.status = QStatusBar()
        self.setStatusBar(self.status)

        self.update_title()
        self.setWindowIcon(
            QIcon(os.path.join(skippy.config.ASSETS_FOLDER, "skippy.ico"))
        )
        self.resize(self.settings.value("size", QSize(700, 700)))
        self.move(self.settings.value("pos", QPoint(200, 200)))
        self.setWindowState(
            Qt.WindowState(self.settings.value("state", Qt.WindowNoState))
        )
        self.font = QFont("Arial", 10)
        self.setFont(self.font)

    def update_title(self):
        self.setWindowTitle(
            f"{self.tab.tabs.tabText(self.tab.tabs.currentIndex())} | skippy - {skippy.config.version}"
        )

    @skippy.utils.critical.critical
    def download(self, *args):
        dDialog = DownloadDialog(self)

    @skippy.utils.critical.critical
    def upload(self, widget):
        if widget.data["parent"] != None:
            wiki = pyscp.wikidot.Wiki(widget.data["parent"][0])
            profile = skippy.utils.profile.Profile.load()
            wiki.auth(profile[0], profile[1])
            p = wiki(widget.data["parent"][1])
            p.edit(
                source=widget.data["source"],
                title=widget.data["title"],
                comment="Edit using Skippy",
            )
            p.set_tags(widget.data["tags"])
            for file in widget.data["files"]:
                try:
                    p.upload(file, base64.b64decode(widget.data["files"][file]))
                    log.debug(f"File {file} is uploaded")
                except RuntimeError as e:
                    log.debug(f"RuntimeError({str(e)}): File {file} isn't uploaded")
            for file in p.files:
                if file.name not in widget.data["files"]:
                    p.remove_file(file.name)
                    log.debug(f"File {file.name} is deleted")
            log.debug(
                f"""Upload "{widget.data['title']}" to "{"/".join(widget.data["parent"])}" """
            )
        else:
            self.upload_as(widget)

    @skippy.utils.critical.critical
    def upload_as(self, widget):
        uDialog = UploadDialog(widget, self)

    @skippy.utils.critical.critical
    def login(self, *args):
        lDialog = LoginDialog(self)

    @skippy.utils.critical.critical
    def logout(self, *args):
        skippy.utils.profile.Profile.save("", "")
        self.hide()
        lDialod = LoginDialog(self)
        log.debug(f"Logout")

    @skippy.utils.critical.critical
    def save_session(self, *args):
        path, _ = QFileDialog.getSaveFileName(
            self, "Save file", "", "JSON file (*.json)\nAll files (*.*)"
        )
        if path != "":
            self.session.save(path)

    @skippy.utils.critical.critical
    def load_session(self, *args):
        path, _ = QFileDialog.getOpenFileName(
            self, "Save file", "", "JSON file (*.json)\nAll files (*.*)"
        )
        if path != "":
            self.session.load(path)
        self.session.save()

    @skippy.utils.critical.critical
    def toggle_theme(self, *args):
        if self.settings.value("mode", "light") == "light":
            log.debug("Dark mode now")
            self.settings.setValue("mode", "dark")
            qtmodern.styles.dark(self.app)
        else:
            log.debug("Light mode now")
            self.settings.setValue("mode", "light")
            qtmodern.styles.light(self.app)
        for i in self.menus.action_list:
            i.setIcon(
                QIcon(
                    os.path.join(
                        skippy.config.ASSETS_FOLDER,
                        self.settings.value("mode", "light"),
                        f"{self.menus.action_list[i]}.png",
                    )
                )
            )

    def updateTranslate(self, lang):
        self.settings.setValue("lang", lang)
        self.restart()

    def restart(self):
        self.close()
        self.__class__(self.app).show()

    def contextMenuEvent(self, event):
        self.menus.contextMenu.exec_(self.mapToGlobal(event.pos()))

    def resizeEvent(self, event):
        self.login_status.move(self.width() - 200, -14)
        if self.width() < 350:
            self.login_status.hide()
        else:
            self.login_status.show()

    def closeEvent(self, e):
        self.settings.setValue("size", self.size())
        self.settings.setValue("pos", self.pos())
        self.settings.setValue("state", self.windowState())

        e.accept()


def start_ui():
    app = QApplication(sys.argv)
    app.setApplicationName("skippy")

    window = App(app)

    if window.settings.value("mode", "light") == "dark":
        qtmodern.styles.dark(app)
    else:
        qtmodern.styles.light(app)

    if (
        skippy.utils.profile.Profile.load()[0] == ""
        or skippy.utils.profile.Profile.load()[1] == ""
    ):
        lDialod = LoginDialog(window)
    else:
        window.show()

    log.info("Skippy was started...")

    app.exec_()
    window.session.save()



from abc import ABCMeta, abstractmethod

try:
    from collections.abc import namedtuple
except ImportError:
    from collections import namedtuple

Field = namedtuple("Field", "tag name description")


class Elements:
    elements = []

    @classmethod
    def register(cls, generator):
        cls.elements.append(generator)


#############################
# Elements Base
#############################
class BaseElement(metaclass=ABCMeta):
    __alias__ = ""
    __description__ = ""

    base = ""

    def __init__(self):
        self.requiredFieldList = []
        self.requiredFields()

    def checkRequiredArgs(self, args):
        for field in self.requiredFieldList:
            if field.tag not in args:
                raise TypeError(
                    f"{self.__class__.__name__}.generate() don't received required argument: {field.tag}"
                )

    @abstractmethod
    def requiredFields(self):
        pass

    def addField(self, tag, name="", description=""):
        self.requiredFieldList.append(Field(tag, name, description))

    def prepareArgs(self, args):
        return args

    def generate(self, **args):
        self.checkRequiredArgs(args)
        args = self.prepareArgs(args)

        data = self.base
        for name in args:
            data = data.replace(f"<<{name}>>", args[name])
        return data


class BaseComponent(BaseElement):
    component = ""
    base = "[[include <<component>> <<args>>]]"

    def prepareArgs(self, args):
        return {
            "component": self.component,
            "args": " |\n".join([f"{arg} = {args[arg]}" for arg in args if args[arg]]),
        }


class BaseModule(BaseElement):
    module = ""
    base = "[[module <<module>> <<args>>]]\n<<content>>\n[[/module]]"

    def __init__(self):
        self.requiredFieldList = []
        self.addField("content", "Content", f"Module {self.module} Content")
        self.requiredFields()

    def prepareArgs(self, args):
        return {
            "module": self.module,
            "content": args["content"],
            "args": " ".join(
                [
                    f'{arg}="{args[arg]}"'
                    for arg in args
                    if arg != "content" and args[arg]
                ]
            ),
        }


#############################
# Wikidot Modules
#############################
class ModuleCSS(BaseModule):
    __alias__ = "Module CSS"
    __description__ = "lets you insert CSS code into a wiki page. This is particularly useful for cross-site include (CSI) packages that need to use custom styling for their code. When you use the CSS module in a CSI, that CSS code will be included in all pages that use the CSI."

    module = "CSS"

    def requiredFields(self):
        self.addField(
            "show",
            "Show CSS Code",
            'You can render the module`s CSS code on the page in a [[code type="css"]] block by adding: show="true"',
        )
        self.addField(
            "disable",
            "Disable Module CSS",
            'You can disable the module`s CSS code so that it doesn`t affect the theme by adding: disable="true"',
        )


class ModuleListUsers(BaseModule):
    __alias__ = "Module ListUsers"
    __description__ = "produces formatted output that lets you report on a set of users working with a site."

    module = "ListUsers"

    def requiredFields(self):
        pass

    def prepareArgs(self, args):
        return {"module": self.module, "content": args["content"], "args": 'users="."'}


class ModuleListPages(BaseModule):
    __alias__ = "Module ListPages"
    __description__ = "is a general-purpose and widely-used tool that selects and display pages within a site."

    module = "ListPages"

    def requiredFields(self):
        #############################
        # Ordering pages
        #############################
        self.addField("order", "Order", "Specify order criteria")
        #############################
        # Pagination
        #############################
        self.addField("limit", "Limit", "Limit total items")
        self.addField("perPage", "Per page", "Limit per pagination")
        self.addField("reverse", "Reverse", "Show pages in reversed order")
        #############################
        # Module body
        #############################
        self.addField("separate", "Separate", "Separation specifier")
        self.addField("wrapper", "Wrapper", "Wrapper specifier")
        self.addField("prependLine", "Prepend Line", "Specify order criteria")
        self.addField("appendLine", "Append Line", "Footer specifier")
        #############################
        # Selecting pages
        #############################
        self.addField("pagetype", "Page type", "Select by type of page")
        self.addField("category", "Category", "Select by category")
        self.addField("tags", "Tags", "Select by tags")
        self.addField("parent", "Parent", "Select by parent page")
        self.addField("link_to", "Outgoing links", "Select by outgoing links")
        self.addField("created_at", "Created at", "Select by date of creation")
        self.addField("updated_at", "Updated at", "Select by date of update")
        self.addField("created_by", "Author", "Select by original author")
        self.addField("rating", "Rating", "Select by rating")
        self.addField("votes", "Votes", "Select by number of votes")
        self.addField("offset", "Offset", "Start list after an offset of pages")
        self.addField("range", "Range", "Select a range of pages")
        self.addField("name", "Page name", "Select by page name")
        self.addField("fullname", "Page fullname", "Select by fullname")


Elements.register(ModuleCSS)
Elements.register(ModuleListPages)
Elements.register(ModuleListUsers)
